@@ -10,7 +10,8 @@ from beancount.parser import printer, parser
 
 from beancount_bot.config import get_global, GLOBAL_MANAGER, get_config
 from beancount_bot.dispatcher import Dispatcher
-from beancount_bot.util import load_class
+from beancount_bot.i18n import _
+from beancount_bot.util import load_class, stringify_errors, logger
 
 META_UUID = 'tgbot_uuid'
 META_TIME = 'tgbot_time'
@@ -31,12 +32,15 @@ class TransactionManager:
         self.dispatchers = dispatchers
         self.__bean_file = bean_file
 
-    def create(self, tx: Union[Transaction, str]) -> Tuple[Uuid, Union[Transaction, str]]:
+    def create(self, tx: Union[Transaction, str], add_tags=None) -> Tuple[Uuid, Union[Transaction, str]]:
         """
         创建交易
         :param tx:
+        :param add_tags: 给交易添加的标签
         :return:
         """
+        if add_tags is None:
+            add_tags = []
         tx_uuid = Uuid(uuid.uuid4())
         if isinstance(tx, str):
             # 保存至账本
@@ -48,7 +52,10 @@ class TransactionManager:
             tx = copy.deepcopy(tx)
             tx.meta[META_UUID] = tx_uuid
             tx.meta[META_TIME] = str(datetime.datetime.now())
+            # 添加标签
+            tx = tx._replace(tags=set(tx.tags).union(add_tags))
             # 保存至账本
+            logger.debug("创建交易：%s", tx)
             with open(self.bean_file, 'a+', encoding='utf-8') as f:
                 printer.print_entry(tx, file=f)
             return tx_uuid, tx
@@ -61,18 +68,23 @@ class TransactionManager:
         :param tx_uuid:
         :return:
         """
-        entries, errors, _ = parser.parse_file(self.bean_file)
-        if len(errors) > 0:
-            desc = '\n'.join(map(lambda err: f'行 {err.source["lineno"]}：{err.message}', errors))
-            raise ValueError("账本文件内容错误！\n" + desc)
+        entries, errors, __ = parser.parse_file(self.bean_file)
         # 筛选交易
         to_delete = next(
             filter(lambda tx: tx.meta[META_UUID] == tx_uuid if META_UUID in tx.meta else False, entries),
             None
         )
         if to_delete is None:
-            # 可能是非交易语句
-            return self._remove_comment_wrapped(tx_uuid)
+            # 不存在此交易，可能是非交易语句
+            try:
+                return self._remove_comment_wrapped(tx_uuid)
+            except Exception as e:
+                # 如果账本文件解析错误，报解析错误
+                if len(errors) > 0:
+                    desc = stringify_errors(errors)
+                    raise ValueError(_("账本文件内容错误或交易不存在！\n{desc}").format(desc=desc))
+                # 账本文件正常，报其他
+                raise e
         # 统计删除行。避免删除其他语句。
         min_line = to_delete.meta['lineno']
         max_line = min_line
@@ -105,20 +117,20 @@ class TransactionManager:
                 max_line = i
                 break
         if min_line == -1 or max_line == -1:
-            raise ValueError("交易不存在！")
+            raise ValueError(_("交易不存在！"))
         # 删除
         with open(self.bean_file, 'w', encoding='utf-8') as f:
             f.write(''.join(lines[:min_line] + lines[max_line + 1:]))
         return ''.join(lines[min_line + 1:max_line])[:-1]
 
-    def create_from_str(self, tx_str) -> Union[Tuple[Uuid, Transaction], Tuple[None, str]]:
+    def create_from_str(self, tx_str, **kwargs) -> Union[Tuple[Uuid, Transaction], Tuple[None, str]]:
         """
         从交易语法创建交易
         :param tx_str:
         :return:
         """
         tx = self._parse_transaction(tx_str)
-        tx_uuid, _ = self.create(tx)
+        tx_uuid, _ = self.create(tx, **kwargs)
         return tx_uuid, tx
 
     def _parse_transaction(self, tx_str) -> Transaction:
@@ -134,7 +146,7 @@ class TransactionManager:
                 continue
         else:
             # 没有匹配
-            raise ValueError("无法识别此交易语法")
+            raise ValueError(_("无法识别此交易语法"))
 
     @property
     def bean_file(self) -> str:
