@@ -71,6 +71,8 @@ class TestTransactionManager(unittest.TestCase):
             post = str(uuid.uuid4())
             f.write(f'; {post}\n')
         # 删除
+        self.assertRaises(ValueError, manager.remove, uuid.uuid4())
+
         delete_tx = manager.remove(tx_uuid)
         self.assertEqual(tx_uuid, delete_tx.meta[transaction.META_UUID])
 
@@ -95,6 +97,8 @@ class TestTransactionManager(unittest.TestCase):
             post = str(uuid.uuid4())
             f.write(f'; {post}\n')
         # 删除
+        self.assertRaises(ValueError, manager.remove, uuid.uuid4())
+
         delete_tx = manager.remove(tx_uuid)
         self.assertEqual(tx, delete_tx)
 
@@ -105,3 +109,35 @@ class TestTransactionManager(unittest.TestCase):
         self.assertNotIn(tx_uuid, data)
         self.assertIn(pre, data)
         self.assertIn(post, data)
+
+    def test_remove_wrong_syntax_tolerance(self):
+        # Mock
+        poison_str = str(uuid.uuid4())
+
+        class MockDispatcher(Dispatcher):
+            def _process_raw(self, input_str: str) -> str:
+                return f'''
+                2010-01-01 * "Payee" "{poison_str}"
+                  Income:Unknown
+                  Assets:Unknown  1 CNY
+                '''
+
+        # 添加语法错误内容
+        manager = TransactionManager([MockDispatcher()], self.tmp_file)
+        with open(self.tmp_file, 'a+', encoding='utf-8') as f:
+            f.write(f'wrong syntax\n')
+        # 添加正确语法交易
+        correct_tx, _ = manager.create_from_str('')
+        # 可以正常删除
+        self.assertRaises(ValueError, manager.remove, uuid.uuid4())
+        delete_tx = manager.remove(correct_tx)
+        self.assertEqual(correct_tx, delete_tx.meta[transaction.META_UUID])
+
+        with open(self.tmp_file, 'r', encoding='utf-8') as f:
+            data = f.read()
+        self.assertNotIn(poison_str, data)
+        self.assertNotIn(correct_tx, data)
+        self.assertIn('wrong syntax', data)
+
+        with open(self.tmp_file, 'w', encoding='utf-8') as f:
+            f.write(data.replace('wrong syntax', ''))
